const Immutable = require('immutable')
const Task = require('data.task')
const Either = require('data.either')

const firstVariant = component =>
  Either.fromNullable(component.get('variants').first())

const {
  createValidator
} = require('@salesforce-ux/design-system-markup/server')

module.exports = (parser, delegates) => {
  const validate = createValidator(parser.comments)
  // setNewComponent :: (State, {}) -> Task Err State
  const setNewComponent = (state, payload) =>
    parser
      .component(payload.get('component'))
      .fold(Task.rejected, Task.of)
      .map(component => state
        .set('component', component)
        .set('variant', firstVariant(component).getOrElse(null))
        .set('variants', component.get('variants').map(v => v.get('id')))
        .set('modifiers', Immutable.List())
      )
      .chain(state =>
        state.has('variant')
          ? setExample(state)
          : Task.of(state)
      )
  // setExample :: State -> Task Err State
  const setExample = (state) =>
    delegates.getMarkup(
      state.getIn(['component', 'id']),
      state.getIn(['variant', 'id'])
    )
    .map(Immutable.fromJS)
    .map(sections =>
      sections.map(section =>
        section.update('items', items =>
          items.map(item =>
            item.set('errors', validate(item.get('markup')))
          )
        )
      )
    )
    .map(sections =>
      state
      .set('markup', sections)
      .set('modifiers', Immutable.List())
    )
  // reducer :: (State, {type: String, payload: a}) -> Task(State)
  const reducer = (state, action) => {
    const type = action.get('type')
    const payload = action.get('payload')
    switch (type) {
      case 'initialize':
        return Task.of(Immutable.Map({
          components: parser
            .components()
            .map(c => c.get('id')),
          variants: Immutable.List()
        }))
      case 'componentSelected':
        return setNewComponent(state, payload)
      case 'variantSelected':
        return Either
          .of(state)
          .chain(state =>
            parser
            .findVariant(state.get('component'), payload.get('variant'))
            .map(variant =>
              state
              .set('variant', variant)
            )
          )
          .fold(Task.rejected, Task.of)
          .chain(setExample)
      case 'modifierSelected':
        return Task.of(state)
          .map(state =>
            state.set('modifiers', payload.get('modifiers'))
          )
      case 'cssChanged':
        return Task.of(state)
          .map(s =>
            s.set('styles', Date.now())
          )
      case 'commentsChanged':
        return Task.of(state)
          .chain(s =>
            reducer(s, Immutable.Map({
              type: 'componentSelected',
              payload: Immutable.Map({component: state.getIn(['component', 'id'])})
            }))
          )
          .chain(s =>
            reducer(s, Immutable.Map({
              type: 'variantSelected',
              payload: Immutable.Map({variant: state.getIn(['variant', 'id'])})
            }))
          )
          .chain(s =>
            reducer(s, Immutable.Map({
              type: 'modifierSelected',
              payload: Immutable.Map({modifiers: state.get('modifiers')})
            }))
          )
      default:
        return Task.of(state)
    }
  }
  return reducer
}
